@@ -151,7 +151,8 @@ MathStructure &MathStructure::getRange(int start, int end, MathStructure &mstruc
 
 void MathStructure::resizeVector(size_t i, const MathStructure &mfill) {
 	if(i > SIZE) {
-		while(i > SIZE) {
+		for(size_t index = SIZE; index < i; index++) {
+			if(index % 10002 == 10001 && CALCULATOR->aborted()) return;
 			APPEND(mfill);
 		}
 	} else if(i < SIZE) {
@@ -264,10 +265,10 @@ void MathStructure::setElement(const MathStructure &mstruct, size_t row, size_t
 void MathStructure::addRows(size_t r, const MathStructure &mfill) {
 	if(r == 0) return;
 	size_t cols = columns();
-	MathStructure mstruct; mstruct.clearVector();
-	mstruct.resizeVector(cols, mfill);
 	for(size_t i = 0; i < r; i++) {
-		APPEND(mstruct);
+		APPEND(m_zero)
+		LAST.clearVector();
+		for(size_t i2 = 0; i2 < cols; i2++) LAST.addChild(mfill);
 	}
 }
 void MathStructure::addColumns(size_t c, const MathStructure &mfill) {
@@ -703,7 +704,7 @@ bool MathStructure::invertMatrix(const EvaluationOptions &eo) {
 				for (i = d + 1; i < n; i++) {
 					if(!mtrx[i][d].isZero()) break;
 				}
-
+				if(CALCULATOR->aborted()) return false;
 				if(i == n) {
 					CALCULATOR->error(true, _("Inverse of singular matrix."), NULL);
 					return false;
@@ -727,7 +728,7 @@ bool MathStructure::invertMatrix(const EvaluationOptions &eo) {
 			mtmp.recip();
 
 			for(j = 0; j < n; j++) {
-
+				if(CALCULATOR->aborted()) return false;
 				if(j > d) {
 					mtrx[d][j].number() *= mtmp;
 				}
@@ -744,7 +745,7 @@ bool MathStructure::invertMatrix(const EvaluationOptions &eo) {
 				mtmp.negate();
 
 				for(j = 0; j < n; j++) {
-
+					if(CALCULATOR->aborted()) return false;
 					if(j > d) {
 						mtrx[i][j].number() += mtrx[d][j].number() * mtmp;
 					}
@@ -773,6 +774,7 @@ bool MathStructure::adjointMatrix(const EvaluationOptions &eo) {
 	MathStructure msave(*this);
 	for(size_t index_r = 0; index_r < SIZE; index_r++) {
 		for(size_t index_c = 0; index_c < CHILD(0).size(); index_c++) {
+			if(CALCULATOR->aborted()) return false;
 			msave.cofactor(index_r + 1, index_c + 1, CHILD(index_r)[index_c], eo);
 		}
 	}
@@ -790,6 +792,7 @@ bool MathStructure::transposeMatrix() {
 	resizeMatrix(CHILD(0).size(), SIZE, m_undefined);
 	for(size_t index_r = 0; index_r < SIZE; index_r++) {
 		for(size_t index_c = 0; index_c < CHILD(0).size(); index_c++) {
+			if(CALCULATOR->aborted()) return false;
 			CHILD(index_r)[index_c] = msave[index_c][index_r];
 		}
 	}
