@@ -17,7 +17,9 @@
 
 #include <string.h>
 #include <time.h>
-#include <iconv.h>
+#ifdef HAVE_ICONV
+#	include <iconv.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sstream>
@@ -304,7 +306,7 @@ int gcd(int i1, int i2) {
 size_t unicode_length(const string &str) {
 	size_t l = str.length(), l2 = 0;
 	for(size_t i = 0; i < l; i++) {
-		if(str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
+		if((signed char) str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
 			l2++;
 		}
 	}
@@ -313,7 +315,7 @@ size_t unicode_length(const string &str) {
 size_t unicode_length(const char *str) {
 	size_t l = strlen(str), l2 = 0;
 	for(size_t i = 0; i < l; i++) {
-		if(str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
+		if((signed char) str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
 			l2++;
 		}
 	}
@@ -323,9 +325,9 @@ size_t unicode_length(const char *str) {
 bool text_length_is_one(const string &str) {
 	if(str.empty()) return false;
 	if(str.length() == 1) return true;
-	if(str[0] >= 0) return false;
+	if((signed char) str[0] >= 0) return false;
 	for(size_t i = 1; i < str.length(); i++) {
-		if(str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
+		if((signed char) str[i] > 0 || (unsigned char) str[i] >= 0xC0) {
 			return false;
 		}
 	}
@@ -336,15 +338,15 @@ bool equalsIgnoreCase(const string &str1, const string &str2) {
 	if(str1.empty() || str2.empty()) return false;
 	for(size_t i1 = 0, i2 = 0; i1 < str1.length() || i2 < str2.length(); i1++, i2++) {
 		if(i1 >= str1.length() || i2 >= str2.length()) return false;
-		if((str1[i1] < 0 && i1 + 1 < str1.length()) || (str2[i2] < 0 && i2 + 1 < str2.length())) {
+		if(((signed char) str1[i1] < 0 && i1 + 1 < str1.length()) || ((signed char) str2[i2] < 0 && i2 + 1 < str2.length())) {
 			size_t iu1 = 1, iu2 = 1;
-			if(str1[i1] < 0) {
-				while(iu1 + i1 < str1.length() && str1[i1 + iu1] < 0) {
+			if((signed char) str1[i1] < 0) {
+				while(iu1 + i1 < str1.length() && (signed char) str1[i1 + iu1] < 0) {
 					iu1++;
 				}
 			}
-			if(str2[i2] < 0) {
-				while(iu2 + i2 < str2.length() && str2[i2 + iu2] < 0) {
+			if((signed char) str2[i2] < 0) {
+				while(iu2 + i2 < str2.length() && (signed char) str2[i2 + iu2] < 0) {
 					iu2++;
 				}
 			}
@@ -383,15 +385,15 @@ bool equalsIgnoreCase(const string &str1, const char *str2) {
 	if(str1.empty() || strlen(str2) == 0) return false;
 	for(size_t i1 = 0, i2 = 0; i1 < str1.length() || i2 < strlen(str2); i1++, i2++) {
 		if(i1 >= str1.length() || i2 >= strlen(str2)) return false;
-		if((str1[i1] < 0 && i1 + 1 < str1.length()) || (str2[i2] < 0 && i2 + 1 < strlen(str2))) {
+		if(((signed char) str1[i1] < 0 && i1 + 1 < str1.length()) || ((signed char) str2[i2] < 0 && i2 + 1 < strlen(str2))) {
 			size_t iu1 = 1, iu2 = 1;
-			if(str1[i1] < 0) {
-				while(iu1 + i1 < str1.length() && str1[i1 + iu1] < 0) {
+			if((signed char) str1[i1] < 0) {
+				while(iu1 + i1 < str1.length() && (signed char) str1[i1 + iu1] < 0) {
 					iu1++;
 				}
 			}
-			if(str2[i2] < 0) {
-				while(iu2 + i2 < strlen(str2) && str2[i2 + iu2] < 0) {
+			if((signed char) str2[i2] < 0) {
+				while(iu2 + i2 < strlen(str2) && (signed char) str2[i2 + iu2] < 0) {
 					iu2++;
 				}
 			}
@@ -426,6 +428,61 @@ bool equalsIgnoreCase(const string &str1, const char *str2) {
 	return true;
 }
 
+string sub_suffix_html(const string &name) {
+	size_t i = name.rfind('_');
+	bool b = (i == string::npos || i == name.length() - 1 || i == 0);
+	size_t i2 = 1;
+	string str;
+	if(b) {
+		if(is_in(NUMBERS, name[name.length() - 1])) {
+			while(name.length() > i2 + 1 && is_in(NUMBERS, name[name.length() - 1 - i2])) {
+				i2++;
+			}
+		} else {
+			while((signed char) name[name.length() - i2] < 0 && (unsigned char) name[name.length() - i2] < 0xC0 && i2 < name.length()) {
+				i2++;
+			}
+		}
+		str += name.substr(0, name.length() - i2);
+	} else {
+		str += name.substr(0, i);
+	}
+	if(b) {
+		str += "<sub class=\"nous\">";
+		str += name.substr(name.length() - i2, i2);
+	} else {
+		str += "<sub>";
+		str += name.substr(i + 1, name.length() - (i + 1));
+	}
+	str += "</sub>";
+	return str;
+}
+string sub_suffix(const string &name, const string &tag_begin, const string &tag_end) {
+	size_t i = name.rfind('_');
+	bool b = (i == string::npos || i == name.length() - 1 || i == 0);
+	size_t i2 = 1;
+	string str;
+	if(b) {
+		if(is_in(NUMBERS, name[name.length() - 1])) {
+			while(name.length() > i2 + 1 && is_in(NUMBERS, name[name.length() - 1 - i2])) {
+				i2++;
+			}
+		} else {
+			while((signed char) name[name.length() - i2] < 0 && (unsigned char) name[name.length() - i2] < 0xC0 && i2 < name.length()) {
+				i2++;
+			}
+		}
+		str += name.substr(0, name.length() - i2);
+	} else {
+		str += name.substr(0, i);
+	}
+	str += tag_begin;
+	if(b) str += name.substr(name.length() - i2, i2);
+	else str += name.substr(i + 1, name.length() - (i + 1));
+	str += tag_end;
+	return str;
+}
+
 void parse_qalculate_version(string qalculate_version, int *qalculate_version_numbers) {
 	for(size_t i = 0; i < 3; i++) {
 		size_t dot_i = qalculate_version.find(".");
@@ -610,6 +667,14 @@ string getGlobalDefinitionsDir() {
 	return "";
 #else
 #	ifndef WIN32
+	char buffer[500];
+	if(getcwd(buffer, 500)) {
+		string dir = buffer;
+		if(dirExists(buildPath(dir, "libqalculate")) && fileExists(buildPath(dir, "data", "functions.xml"))) return buildPath(dir, "data");
+		size_t i = dir.rfind("/");
+		if(i != string::npos && i > 0 && i < dir.length() - 1) dir = dir.substr(0, i);
+		if(dirExists(buildPath(dir, "libqalculate")) && fileExists(buildPath(dir, "data", "functions.xml"))) return buildPath(dir, "data");
+	}
 	return string(PACKAGE_DATA_DIR) + "/qalculate";
 #	else
 	char exepath[MAX_PATH];
@@ -727,6 +792,7 @@ bool removeDir(string dirpath) {
 }
 
 char *locale_from_utf8(const char *str) {
+#ifdef HAVE_ICONV
 	iconv_t conv = iconv_open("", "UTF-8");
 	if(conv == (iconv_t) -1) return NULL;
 	size_t inlength = strlen(str);
@@ -740,8 +806,12 @@ char *locale_from_utf8(const char *str) {
 	memset(buffer, 0, 4);
 	if(err == (size_t) -1) {free(dest); return NULL;}
 	return dest;
+#else
+	return NULL;
+#endif
 }
 char *locale_to_utf8(const char *str) {
+#ifdef HAVE_ICONV
 	iconv_t conv = iconv_open("UTF-8", "");
 	if(conv == (iconv_t) -1) return NULL;
 	size_t inlength = strlen(str);
@@ -755,6 +825,9 @@ char *locale_to_utf8(const char *str) {
 	memset(buffer, 0, 4 * sizeof(char));
 	if(err == (size_t) -1) {free(dest); return NULL;}
 	return dest;
+#else
+	return NULL;
+#endif
 }
 char *utf8_strdown(const char *str, int l) {
 #ifdef HAVE_ICU
